library(nimble)
library(coda)



longcode <- nimbleCode({
  cov_tau[1:m[1], 1:m[1], 1] <- E[1:msum[1],1:6] %*% Sigma[1:6, 1:6] %*% t(E[1:msum[1],1:6])
  tau[1,1:m[1]] ~ dmnorm(mu_tau[1:m[1]], cov_tau[1:m[1], 1:m[1], 1])
  
  for (i in 2:N) {
    cov_tau[1:m[i], 1:m[i], i] <- E[(1+msum[i-1]):msum[i], 1:6] %*% Sigma[1:6, 1:6] %*% t(E[(1+msum[i-1]):msum[i], 1:6])
    tau[i,1:m[i]] ~ dmnorm(mu_tau[1:m[i]], cov_tau[1:m[i], 1:m[i], i])
  }
  for (i in 1:N) {
    for (t in 1:m[i]) {
      y[i,t] ~ dbern(p[i,t])
      logit(p[i,t]) <- pb[i,t]
      mu_tau[t] <- 0
      pb[i,t] <- beta0 + beta[1] * x1[i, t] + beta[2] * x2[i,t] + beta[3] * x3[i,t] +
        beta[4] * x4[i,t] + tau[i, t] 
    }
  }
  beta0 ~ dnorm(0, 0.01)
  for (j in 1:4) {
    beta[j] ~ dnorm(0, 0.01)
  }
  Sigma[1:6, 1:6] ~ dinvwish(S[1:6,1:6], df = 7)
})


# simulate data ####
N <- 500
y <- x1 <- x2 <- x3 <- x4 <- matrix(ncol = 6, nrow = N)
for (i in 1:N) {
  for (j in 1:6) {
    y[i,j] ~ rbinom(1, 1, 0.5)
    x2[i,j] ~ rbinom(1, 1, 0.5)
    x3[i,j] ~ rbinom(1, 1, 0.5)
    x1[i,j] ~ rnorm(1, 0, 1)
    x4[i,j] ~ rnorm(1, 0, 1)
  }
}
# waves 
waves <- matrix(ncol = 6, nrow = N)
for (i in 1:N) {
  for (j in 1:6) {
    waves[i,j] <- rbinom(1, 1, 0.5)
  }
  if (sum(waves[i,]) == 0) {waves[i, 6] <- 1}
}
# number of visits
m <- apply(waves, 1, sum)
table(m)

Ematrix.fun <- function(waves) {
  wave.id <- which(waves == 1)
  a <- length(wave.id)
  E <- matrix(0, nrow = a, ncol = 6)
  for (i in 1:a) {
    E[i, wave.id[i]] <- 1
  }
  return(E)
}
#REmatrix <- nimbleRcall(function(x = double(1)){}, Rfun = 'Ematrix.fun',
#                        returnType = double())
Ematrix.fun(waves[2,])

msum <- cumsum(m)
Ematrix <- matrix(nrow = sum(m), ncol = 6)
Ematrix[1:msum[1],] <- Ematrix.fun(waves[1,])
for (i in 2:N) {
  Ematrix[(1+msum[i-1]):(msum[i]),] <- Ematrix.fun(waves[i,])
}


longdata <- list(y = y, x1 = x1, x2 = x2, x3 = x3, x4 = x4,
                 E = Ematrix)
longconsts <- list(N = N, msum = msum, m = m)
tau.inits <- matrix(rnorm(longconsts$N * 6), nrow = longconsts$N, ncol = 6)
longinits <- list(beta0 = 0, beta = rnorm(4), 
                  p = matrix(0.5, nrow = longconsts$N, ncol = 6),
                  pb = matrix(rnorm(longconsts$N * 6), nrow = longconsts$N, ncol = 6),
                  tau = tau.inits, S = diag(c(1:6)), Sigma = diag(6))
longpro <- nimbleModel(code = longcode, name = "longpro", constants = longconsts,
                       data = longdata, inits = longinits)
clong <- compileNimble(longpro)
longconf <- configureMCMC(longpro, print = FALSE)
longconf$addMonitors(c("beta0", "beta", "beta0y", "Sigma"))
longmcmc <- buildMCMC(longconf)
clongmcmc <- compileNimble(longmcmc, project = longpro)
clong$setInits(longinits)
mcmc.out <- runMCMC(clongmcmc, niter = 10000, setSeed = 1234, thin = 2, nchains = 1)
pos_mcmc <- as.mcmc(mcmc.out[-c(1:3000),])
